@@ -28,13 +28,37 @@
 #include "seahorse-gpgmex.h"
 #include "seahorse-operation.h"
 #include "seahorse-hkp-source.h"
-#include "seahorse-multi-source.h"
 #include "seahorse-util.h"
-#include "seahorse-key.h"
-#include "seahorse-key-pair.h"
+#include "seahorse-pgp-key.h"
 
 #ifdef WITH_HKP
 
+/* Override the DEBUG_HKP_ENABLE switch here */
+/* #define DEBUG_HKP_ENABLE 1 */
+
+#ifndef DEBUG_HKP_ENABLE
+#if _DEBUG
+#define DEBUG_HKP_ENABLE 1
+#else
+#define DEBUG_HKP_ENABLE 0
+#endif
+#endif
+
+#if DEBUG_HKP_ENABLE
+#define DEBUG_HKP(x) g_printerr x
+#define DEBUG_HEADER(a,b,c) dump_soup_header (a,b,c)
+#define DEBUG_REQUEST(a) dump_soup_request (a)
+#define DEBUG_RESPONSE(a) dump_soup_response (a)
+#else
+#define DEBUG_HKP(x) 
+#define DEBUG_HEADER(a,b,c)
+#define DEBUG_REQUEST(a)
+#define DEBUG_RESPONSE(a)
+#endif
+
+#define PGP_KEY_BEGIN   "-----BEGIN PGP PUBLIC KEY BLOCK-----"
+#define PGP_KEY_END     "-----END PGP PUBLIC KEY BLOCK-----"
+
 /* Amount of keys to load in a batch */
 #define DEFAULT_LOAD_BATCH 30
 
@@ -54,7 +78,25 @@ get_hkp_error_domain ()
     return q;
 }
 
-#ifdef _DEBUG
+static gchar*
+get_http_server_address (SeahorseKeySource *src)
+{
+    gchar *server, *t;
+
+    g_object_get (src, "key-server", &server, NULL);
+    g_return_val_if_fail (server != NULL, NULL);
+
+    /* If it already has a port then leave it at the default */
+    if (strchr (server, ':'))
+        return server;
+
+    /* Otherwise use default HKP port */
+    t = g_strdup_printf ("%s:11371", server);
+    g_free (server);
+    return t;
+}
+
+#ifdef DEBUG_HKP_ENABLE
 
 static void
 dump_soup_header (const gchar *name, const gchar *value, gpointer user_data)
@@ -98,7 +140,7 @@ dump_soup_response (SoupMessage *msg)
     }    
 }
 
-#endif
+#endif /* DEBUG_HKP_ENABLE */
 
 /* -----------------------------------------------------------------------------
  *  HKP OPERATION     
@@ -114,7 +156,7 @@ dump_soup_response (SoupMessage *msg)
 typedef gboolean (*OpHKPCallback)   (SeahorseOperation *op, SoupMessage *message);
     
 DECLARE_OPERATION (HKP, hkp)
-    SeahorseHKPSource *hsrc;        /* The source to add keys to */
+    SeahorseHKPSource *hsrc;        /* The source  */
     SoupSession *session;           /* The HTTP session */
     guint total;                    /* Number of requests queued */
     guint requests;                 /* Number of requests remaining */
@@ -162,7 +204,7 @@ seahorse_hkp_operation_cancel (SeahorseOperation *operation)
 {
     SeahorseHKPOperation *hop;
     
-    g_return_if_fail (SEAHORSE_IS_HKP_OPERATION (operation));
+    g_assert (SEAHORSE_IS_HKP_OPERATION (operation));
     hop = SEAHORSE_HKP_OPERATION (operation);
     
     if (hop->session != NULL) 
@@ -171,34 +213,6 @@ seahorse_hkp_operation_cancel (SeahorseOperation *operation)
     seahorse_operation_mark_done (operation, TRUE, NULL);
 }
 
-/* Cancels operation and marks the HKP operation as failed */
-static void
-fail_hkp_operation (SeahorseHKPOperation *hop, guint status, const gchar *msg)
-{
-    gchar *t;
-    
-    g_object_get (hop->hsrc, "key-server", &t, NULL);
-    seahorse_operation_mark_done (SEAHORSE_OPERATION (hop), FALSE, 
-            g_error_new (HKP_ERROR_DOMAIN, status, _("Couldn't communicate with '%s': %s"), 
-                         t, msg ? msg : soup_status_get_phrase (status)));
-    g_free (t);
-}
-
-static SeahorseHKPOperation*
-setup_hkp_operation (SeahorseHKPSource *hsrc)
-{
-    SeahorseHKPOperation *hop;
-
-    g_return_val_if_fail (SEAHORSE_IS_HKP_SOURCE (hsrc), NULL);
-    
-    hop = g_object_new (SEAHORSE_TYPE_HKP_OPERATION, NULL);
-    hop->hsrc = hsrc;
-    g_object_ref (hsrc);
-
-    seahorse_operation_mark_start (SEAHORSE_OPERATION (hop));
-    return hop;    
-}
-
 /* Thanks to GnuPG */
 static void
 dehtmlize(gchar *line)
@@ -266,6 +280,57 @@ dehtmlize(gchar *line)
     }
 }
 
+/* Cancels operation and marks the HKP operation as failed */
+static void
+fail_hkp_operation (SeahorseHKPOperation *hop, SoupMessage *msg, const gchar *text)
+{
+    gchar *t, *server;
+    GError *error = NULL;
+
+    g_object_get (hop->hsrc, "key-server", &server, NULL);
+
+    if (text) {
+        error = g_error_new (HKP_ERROR_DOMAIN, msg ? msg->status_code : 0, text);
+
+    } else if (msg) {
+        /* Make the body lower case, and no tags */
+        t = g_strndup (msg->response.body, msg->response.length);
+        dehtmlize (t);        
+        seahorse_util_string_lower (t);
+
+        if (strstr (t, "no keys"))
+            error = g_error_new (HKP_ERROR_DOMAIN, 0, _("No matching keys found on server '%s'."), server);
+        else if (strstr (t, "too many"))
+            error = g_error_new (HKP_ERROR_DOMAIN, 0, _("Search was not specific enough. Server '%s' found too many keys."), server);
+        else 
+            error = g_error_new (HKP_ERROR_DOMAIN, msg->status_code, _("Couldn't communicate with server '%s': %s"),
+                                 server, soup_status_get_phrase (msg->status_code));
+        g_free (t);
+    } else {
+
+        /* We should always have msg or text */
+        g_assert (FALSE);
+    }        
+
+    seahorse_operation_mark_done (SEAHORSE_OPERATION (hop), FALSE, error);
+    g_free (server);
+}
+
+static SeahorseHKPOperation*
+setup_hkp_operation (SeahorseHKPSource *hsrc)
+{
+    SeahorseHKPOperation *hop;
+
+    g_return_val_if_fail (SEAHORSE_IS_HKP_SOURCE (hsrc), NULL);
+    
+    hop = g_object_new (SEAHORSE_TYPE_HKP_OPERATION, NULL);
+    hop->hsrc = hsrc;
+    g_object_ref (hsrc);
+
+    seahorse_operation_mark_start (SEAHORSE_OPERATION (hop));
+    return hop;    
+}
+
 unsigned int
 parse_hkp_date (const gchar *text)
 {
@@ -415,13 +480,11 @@ refresh_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
     GList *keys, *k;
     gchar *t;
     
-#ifdef _DEBUG
-    g_printerr ("[hkp] Search Result:\n");
-    dump_soup_response (msg);
-#endif      
+    DEBUG_HKP (("[hkp] Search Result:\n"));
+    DEBUG_RESPONSE (msg);
     
     if (SOUP_MESSAGE_IS_ERROR (msg)) {
-        fail_hkp_operation (hop, msg->status_code, NULL);
+        fail_hkp_operation (hop, msg, NULL);
         return;
     }
     
@@ -489,19 +552,17 @@ send_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
 {
     gchar *errmsg;
 
-#ifdef _DEBUG
-    g_printerr ("[hkp] Send Result:\n");
-    dump_soup_response (msg);
-#endif      
+    DEBUG_HKP (("[hkp] Send Result:\n"));
+    DEBUG_RESPONSE (msg);
     
     if (SOUP_MESSAGE_IS_ERROR (msg)) {
-        fail_hkp_operation (hop, msg->status_code, NULL);
+        fail_hkp_operation (hop, msg, NULL);
         return;
     }
     
     errmsg = get_send_result (msg->response.body, msg->response.length);
     if (errmsg) {
-        fail_hkp_operation (hop, 0, errmsg);
+        fail_hkp_operation (hop, NULL, errmsg);
         g_free (errmsg);
         return;
     }
@@ -516,10 +577,32 @@ send_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
                                           hop->requests, hop->total);
 }
 
+gboolean
+detect_key (const gchar *text, gint len, const gchar **start, const gchar **end)
+{
+    const gchar *t;
+
+    if (len == -1)
+        len = strlen (text);
+    
+    /* Find the first of the headers */
+    if((t = g_strstr_len (text, len, PGP_KEY_BEGIN)) == NULL)
+        return FALSE;
+    if (start)
+        *start = t;
+        
+    /* Find the end of that block */
+    if((t = g_strstr_len (t, len - (t - text), PGP_KEY_END)) == NULL)
+        return FALSE;
+    if (end)
+        *end = t;
+    
+    return TRUE; 
+}
+
 static void 
 get_callback (SoupMessage *msg, SeahorseHKPOperation *hop) 
 {
-    SeahorseTextType type;
     const gchar *start;
     const gchar *end;
     gpgme_data_t data;
@@ -527,44 +610,40 @@ get_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
     guint len;
     int r;
     
-#ifdef _DEBUG
-    g_printerr ("[hkp] Get Result:\n");
-    dump_soup_response (msg);
-#endif      
+    DEBUG_HKP (("[hkp] Get Result:\n"));
+    DEBUG_RESPONSE (msg);
     
     if (SOUP_MESSAGE_IS_ERROR (msg)) {
-        fail_hkp_operation (hop, msg->status_code, NULL);
+        fail_hkp_operation (hop, msg, NULL);
         return;
     }
     
     end = text = msg->response.body;
     len = msg->response.length;
     
-    do {
+    for (;;) {
 
         len -= end - text;
         text = end;
         
-        type = seahorse_util_detect_text (text, len, &start, &end);
-
+        if(!detect_key (text, len, &start, &end))
+            break;
+        
         /* Any key blocks get written to our result data */
-        if (type == SEAHORSE_TEXT_TYPE_KEY) {
-            data = (gpgme_data_t)g_object_get_data (G_OBJECT (hop), "result");
-            g_return_if_fail (data != NULL);
+        data = (gpgme_data_t)g_object_get_data (G_OBJECT (hop), "result");
+        g_return_if_fail (data != NULL);
             
-            r = gpgme_data_write (data, start, end - start);
-            g_return_if_fail (r != -1);
+        r = gpgme_data_write (data, start, end - start);
+        g_return_if_fail (r != -1);
 
-            r = gpgme_data_write (data, "\n", 1);
-            g_return_if_fail (r != -1);
-        }
-
-    } while (type != SEAHORSE_TEXT_TYPE_NONE);        
+        r = gpgme_data_write (data, "\n", 1);
+        g_return_if_fail (r != -1);
+    }
         
     if (--hop->requests <= 0)
         seahorse_operation_mark_done (SEAHORSE_OPERATION (hop), FALSE, NULL);
     else
-        seahorse_operation_mark_progress (SEAHORSE_OPERATION (hop), _("Uploading keys..."), 
+        seahorse_operation_mark_progress (SEAHORSE_OPERATION (hop), _("Retrieving keys..."), 
                                           hop->requests, hop->total);
 }
 
@@ -576,7 +655,8 @@ get_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
 static void seahorse_hkp_source_class_init (SeahorseHKPSourceClass *klass);
 
 /* SeahorseKeySource methods */
-static SeahorseOperation*  seahorse_hkp_source_refresh    (SeahorseKeySource *src,
+static SeahorseOperation*  seahorse_hkp_source_load       (SeahorseKeySource *src,
+                                                           SeahorseKeySourceLoad load,
                                                            const gchar *key);
 static SeahorseOperation*  seahorse_hkp_source_import     (SeahorseKeySource *sksrc, 
                                                            gpgme_data_t data);
@@ -614,7 +694,7 @@ seahorse_hkp_source_class_init (SeahorseHKPSourceClass *klass)
     SeahorseKeySourceClass *key_class;
    
     key_class = SEAHORSE_KEY_SOURCE_CLASS (klass);
-    key_class->refresh = seahorse_hkp_source_refresh;
+    key_class->load = seahorse_hkp_source_load;
     key_class->import = seahorse_hkp_source_import;
     key_class->export = seahorse_hkp_source_export;
 
@@ -622,7 +702,8 @@ seahorse_hkp_source_class_init (SeahorseHKPSourceClass *klass)
 }
 
 static SeahorseOperation*
-seahorse_hkp_source_refresh (SeahorseKeySource *src, const gchar *key)
+seahorse_hkp_source_load (SeahorseKeySource *src, SeahorseKeySourceLoad load,
+                          const gchar *match)
 {
     SeahorseOperation *op;
     SeahorseHKPOperation *hop;
@@ -630,38 +711,30 @@ seahorse_hkp_source_refresh (SeahorseKeySource *src, const gchar *key)
     gchar *pattern = NULL;
     gchar *t, *uri, *server;
     
-    g_return_val_if_fail (SEAHORSE_IS_KEY_SOURCE (src), NULL);
-    g_return_val_if_fail (SEAHORSE_IS_HKP_SOURCE (src), NULL);
-    g_return_val_if_fail (key != NULL, NULL);
+    g_assert (SEAHORSE_IS_KEY_SOURCE (src));
+    g_assert (SEAHORSE_IS_HKP_SOURCE (src));
     
-    op = parent_class->refresh (src, key);
+    op = parent_class->load (src, load, match);
     if (op != NULL)
         return op;
 
-    /* No way to find new keys */
-    if (g_str_equal (key, SEAHORSE_KEY_SOURCE_NEW))
+    /* No way to find new all or new keys */
+    if (load == SKSRC_LOAD_NEW || load == SKSRC_LOAD_ALL)
         return seahorse_operation_new_complete (NULL);
         
-    /* All keys, parent_class will have cleared */
-    else if(g_str_equal (key, SEAHORSE_KEY_SOURCE_ALL)) {
-        
-        g_object_get (src, "pattern", &pattern, NULL);
-        g_return_val_if_fail (pattern && pattern[0], NULL);
-        
-        t = soup_uri_encode (pattern, "+=/\\()");
-        g_free (pattern);
-        pattern = t;
+    else if(load == SKSRC_LOAD_SEARCH)
+        pattern = soup_uri_encode (match, "+=/\\()");
         
     /* Load a specific key */
-    } else {
-
+    else if(load == SKSRC_LOAD_KEY) 
         /* TODO: Does this actually work? */
-        pattern = soup_uri_encode (key, NULL);    
-    } 
-        
+        pattern = soup_uri_encode (match, NULL);    
+
+    g_return_val_if_fail (pattern != NULL, NULL);
+    
     hop = setup_hkp_operation (SEAHORSE_HKP_SOURCE (src));
     
-    g_object_get (src, "key-server", &server, NULL);
+    server = get_http_server_address (src);
     g_return_val_if_fail (server && server[0], FALSE);
     
     uri = g_strdup_printf ("http://%s/pks/lookup?op=index&search=%s", 
@@ -674,10 +747,8 @@ seahorse_hkp_source_refresh (SeahorseKeySource *src, const gchar *key)
     soup_session_queue_message (hop->session, message, 
                                 (SoupMessageCallbackFn)refresh_callback, hop);
 
-#ifdef _DEBUG
-    g_printerr ("[hkp] Search Request:\n");
-    dump_soup_request (message);
-#endif      
+    DEBUG_HKP (("[hkp] Search Request:\n"));
+    DEBUG_REQUEST (message);
     
     hop->total = hop->requests = 1;
     t = g_strdup_printf (_("Searching for keys on: %s"), server);
@@ -686,8 +757,9 @@ seahorse_hkp_source_refresh (SeahorseKeySource *src, const gchar *key)
 
     g_free (server);    
     
-    seahorse_server_source_set_operation (SEAHORSE_SERVER_SOURCE (src), 
-                                          SEAHORSE_OPERATION (hop));
+    seahorse_server_source_take_operation (SEAHORSE_SERVER_SOURCE (src), 
+                                           SEAHORSE_OPERATION (hop));
+    g_object_ref (hop);
     return SEAHORSE_OPERATION (hop);
 }
 
@@ -703,7 +775,7 @@ seahorse_hkp_source_import (SeahorseKeySource *sksrc, gpgme_data_t data)
     GSList *l;
     guint len;
     
-    g_return_val_if_fail (SEAHORSE_IS_HKP_SOURCE (sksrc), NULL);
+    g_assert (SEAHORSE_IS_HKP_SOURCE (sksrc));
     hsrc = SEAHORSE_HKP_SOURCE (sksrc);
     
     for (;;) {
@@ -724,7 +796,7 @@ seahorse_hkp_source_import (SeahorseKeySource *sksrc, gpgme_data_t data)
         return seahorse_operation_new_complete (NULL);
     
     /* Figure out the URI we're sending to */    
-    g_object_get (hsrc, "key-server", &server, NULL);
+    server = get_http_server_address (sksrc);
     g_return_val_if_fail (server && server[0], FALSE);
     uri = g_strdup_printf ("http://%s/pks/add", server);
 
@@ -733,7 +805,7 @@ seahorse_hkp_source_import (SeahorseKeySource *sksrc, gpgme_data_t data)
     hop = setup_hkp_operation (hsrc);
     
     for (l = keydata; l; l = g_slist_next (l)) {
-        g_return_val_if_fail (l->data != NULL, FALSE);
+        g_assert (l->data != NULL);
         t = soup_uri_encode((gchar*)(l->data), "+=/\\()");
 
         key = g_strdup_printf ("keytext=%s", t);
@@ -747,10 +819,8 @@ seahorse_hkp_source_import (SeahorseKeySource *sksrc, gpgme_data_t data)
                                     (SoupMessageCallbackFn)send_callback, hop);
         hop->requests++;
 
-#ifdef _DEBUG
-        g_printerr ("[hkp] Send Request:\n");
-        dump_soup_request (message);
-#endif      
+        DEBUG_HKP (("[hkp] Send Request:\n"));
+        DEBUG_REQUEST (message);
     }
 
     hop->total = hop->requests;
@@ -783,7 +853,7 @@ seahorse_hkp_source_export (SeahorseKeySource *sksrc, GList *keys,
     if (g_list_length (keys) == 0)
         return seahorse_operation_new_complete (NULL);
 
-    g_object_get (hsrc, "key-server", &server, NULL);
+    server = get_http_server_address (sksrc);
     g_return_val_if_fail (server && server[0], FALSE);
 
     /* New operation started */    
@@ -801,10 +871,10 @@ seahorse_hkp_source_export (SeahorseKeySource *sksrc, GList *keys,
     }
     
     for ( ; keys; keys = g_list_next (keys)) {
-        g_return_val_if_fail (SEAHORSE_IS_KEY (keys->data), NULL);
+        g_assert (SEAHORSE_IS_KEY (keys->data));
 
         /* Get the key id and limit it to 8 characters */
-        fpr = seahorse_key_get_id (SEAHORSE_KEY (keys->data)->key);
+        fpr = seahorse_key_get_keyid (SEAHORSE_KEY (keys->data));
         l = strlen (fpr);
         if (l > 8)
             fpr += (l - 8);
@@ -821,10 +891,8 @@ seahorse_hkp_source_export (SeahorseKeySource *sksrc, GList *keys,
 
         hop->requests++;
 
-#ifdef _DEBUG
-        g_printerr ("[hkp] Get Request:\n");
-        dump_soup_request (message);
-#endif              
+        DEBUG_HKP (("[hkp] Get Request:\n"));
+        DEBUG_REQUEST (message);
     }
     
     hop->total = hop->requests;
@@ -838,28 +906,19 @@ seahorse_hkp_source_export (SeahorseKeySource *sksrc, GList *keys,
 
 /**
  * seahorse_hkp_source_new
- * @locsrc: Local source to base this key source off of
- * @server: The server to connect to 
- * @pattern: The pattern to search for
+ * @uri: The server to connect to 
  * 
  * Creates a new key source for an HKP PGP server.
  * 
  * Returns: A new HKP Key Source
  */
 SeahorseHKPSource*   
-seahorse_hkp_source_new (SeahorseKeySource *locsrc, const gchar *server,
-                          const gchar *pattern)
+seahorse_hkp_source_new (const gchar *uri, const gchar *host)
 {
-    SeahorseHKPSource *hsrc;
-    
-    g_return_val_if_fail (SEAHORSE_IS_KEY_SOURCE (locsrc) && 
-                          !SEAHORSE_IS_SERVER_SOURCE (locsrc), NULL);
-    g_return_val_if_fail (server && server[0], NULL);
-    
-    hsrc = g_object_new (SEAHORSE_TYPE_HKP_SOURCE, "local-source", locsrc,
-                         "key-server", server, "pattern", pattern, NULL);
-
-    return hsrc;  
+    g_return_val_if_fail (seahorse_hkp_is_valid_uri (uri), NULL);
+    g_return_val_if_fail (host && *host, NULL);
+    return g_object_new (SEAHORSE_TYPE_HKP_SOURCE, "uri", uri,
+                         "key-server", host, NULL);
 }
 
 /**
@@ -875,6 +934,8 @@ seahorse_hkp_is_valid_uri (const gchar *uri)
     gboolean ret = FALSE;
     gchar *t;
     
+    g_return_val_if_fail (uri && uri[0], FALSE);
+    
     /* Replace 'hkp' with 'http' at the beginning of the URI */
     if (strncasecmp (uri, "hkp:", 4) == 0) {
         t = g_strdup_printf("http:%s", uri + 4);
