@@ -28,13 +28,30 @@
 #include "seahorse-gpgmex.h"
 #include "seahorse-operation.h"
 #include "seahorse-ldap-source.h"
-#include "seahorse-multi-source.h"
 #include "seahorse-util.h"
-#include "seahorse-key.h"
-#include "seahorse-key-pair.h"
+#include "seahorse-pgp-key.h"
 
 #ifdef WITH_LDAP
 
+/* Override the DEBUG_LDAP_ENABLE switch here */
+/* #define DEBUG_LDAP_ENABLE 1 */
+
+#ifndef DEBUG_LDAP_ENABLE
+#if _DEBUG
+#define DEBUG_LDAP_ENABLE 1
+#else
+#define DEBUG_LDAP_ENABLE 0
+#endif
+#endif
+
+#if DEBUG_LDAP_ENABLE
+#define DEBUG_LDAP(x) g_printerr x
+#define DEBUG_LDAP_ENTRY(a,b) dump_ldap_entry(a,b)
+#else
+#define DEBUG_LDAP(x) 
+#define DEBUG_LDAP_ENTRY(a,b) 
+#endif
+
 /* Amount of keys to load in a batch */
 #define DEFAULT_LOAD_BATCH 30
 
@@ -92,7 +109,7 @@ get_ldap_server_info (SeahorseLDAPSource *lsrc, gboolean force)
  
 #define LDAP_ERROR_DOMAIN (get_ldap_error_domain())
 
-#ifdef _DEBUG
+#if DEBUG_LDAP_ENABLE
 
 static void
 dump_ldap_entry (LDAP *ld, LDAPMessage *res)
@@ -234,7 +251,7 @@ escape_ldap_value (const gchar *v)
 {
     GString *value;
     
-    g_return_val_if_fail (v, "");
+    g_assert (v);
     value = g_string_sized_new (strlen(v));
     
     for ( ; *v; v++) {
@@ -271,7 +288,7 @@ escape_ldap_value (const gchar *v)
 typedef gboolean (*OpLDAPCallback)   (SeahorseOperation *op, LDAPMessage *result);
     
 DECLARE_OPERATION (LDAP, ldap)
-    SeahorseLDAPSource *lsrc;       /* The source to add keys to */
+    SeahorseLDAPSource *lsrc;       /* The source */
     LDAP *ldap;                     /* The LDAP connection */
     int ldap_op;                    /* The current LDAP async msg */
     guint stag;                     /* The tag for the idle event source */
@@ -340,7 +357,7 @@ seahorse_ldap_operation_cancel (SeahorseOperation *operation)
 {
     SeahorseLDAPOperation *lop;
     
-    g_return_if_fail (SEAHORSE_IS_LDAP_OPERATION (operation));
+    g_assert (SEAHORSE_IS_LDAP_OPERATION (operation));
     lop = SEAHORSE_LDAP_OPERATION (operation);
     
     if (lop->ldap_op != -1) {
@@ -387,9 +404,9 @@ result_callback (SeahorseLDAPOperation *lop)
     gboolean ret;
     int r, i;
     
-    g_return_val_if_fail (SEAHORSE_IS_LDAP_OPERATION (lop), FALSE);
-    g_return_val_if_fail (lop->ldap != NULL, FALSE);
-    g_return_val_if_fail (lop->ldap_op != -1, FALSE);
+    g_assert (SEAHORSE_IS_LDAP_OPERATION (lop));
+    g_assert (lop->ldap != NULL);
+    g_assert (lop->ldap_op != -1);
     
     for (i = 0; i < DEFAULT_LOAD_BATCH; i++) {
      
@@ -436,7 +453,7 @@ done_info_start_op (SeahorseOperation *op, LDAPMessage *result)
     int code;
     int r;
     
-    g_return_val_if_fail (SEAHORSE_IS_LDAP_OPERATION (lop), FALSE);
+    g_assert (SEAHORSE_IS_LDAP_OPERATION (lop));
 
     /* This can be null when we short-circuit the server info */
     if (result) {
@@ -445,10 +462,10 @@ done_info_start_op (SeahorseOperation *op, LDAPMessage *result)
         
         /* If we have results then fill in the server info */
         if (r == LDAP_RES_SEARCH_ENTRY) {
-#ifdef _DEBUG
-            g_printerr ("[ldap] Server Info Result:\n");
-            dump_ldap_entry (lop->ldap, result);
-#endif      
+
+            DEBUG_LDAP (("[ldap] Server Info Result:\n"));
+            DEBUG_LDAP_ENTRY (lop->ldap, result);
+
             /* NOTE: When adding attributes here make sure to add them to kServerAttributes */
             sinfo = g_new0 (LDAPServerInfo, 1);
             sinfo->version = get_int_attribute (lop->ldap, result, "version");
@@ -533,7 +550,7 @@ seahorse_ldap_operation_start (SeahorseLDAPSource *lsrc, OpLDAPCallback cb,
     gchar *server = NULL;
     gchar *t;
 
-    g_return_val_if_fail (SEAHORSE_IS_LDAP_SOURCE (lsrc), NULL);
+    g_assert (SEAHORSE_IS_LDAP_SOURCE (lsrc));
     
     lop = g_object_new (SEAHORSE_TYPE_LDAP_OPERATION, NULL);
     lop->lsrc = lsrc;
@@ -597,7 +614,7 @@ parse_key_from_ldap_entry (SeahorseLDAPOperation *lop, LDAPMessage *res)
     guint flags = 0;
     int length;
         
-    g_return_if_fail (SEAHORSE_IS_LDAP_OPERATION (lop));
+    g_assert (SEAHORSE_IS_LDAP_OPERATION (lop));
     g_return_if_fail (res && ldap_msgtype (res) == LDAP_RES_SEARCH_ENTRY);  
     
     fpr = get_string_attribute (lop->ldap, res, "pgpcertid");
@@ -637,10 +654,10 @@ search_entry (SeahorseOperation *op, LDAPMessage *result)
      
     /* An LDAP entry */
     if (r == LDAP_RES_SEARCH_ENTRY) {
-#ifdef _DEBUG
-            g_printerr ("[ldap] Retrieved Key Entry:\n");
-            dump_ldap_entry (lop->ldap, result);
-#endif      
+        
+        DEBUG_LDAP (("[ldap] Retrieved Key Entry:\n"));
+        DEBUG_LDAP_ENTRY (lop->ldap, result);
+        
         parse_key_from_ldap_entry (lop, result);
         return TRUE;
         
@@ -712,7 +729,7 @@ start_search_operation (SeahorseLDAPSource *lsrc, const gchar *pattern)
     gchar *filter;
     gchar *t;
     
-    g_return_val_if_fail (pattern && pattern[0], NULL);
+    g_assert (pattern && pattern[0]);
     
     t = escape_ldap_value (pattern);
     filter = g_strdup_printf ("(pgpuserid=*%s*)", t);
@@ -737,7 +754,7 @@ start_search_operation_fpr (SeahorseLDAPSource *lsrc, const gchar *fpr)
     gchar *filter, *t;
     guint l;
     
-    g_return_val_if_fail (fpr && fpr[0], NULL);
+    g_assert (fpr && fpr[0]);
     
     l = strlen (fpr);
     if (l > 16)
@@ -782,10 +799,9 @@ get_callback (SeahorseOperation *op, LDAPMessage *result)
     /* An LDAP Entry */
     if (r == LDAP_RES_SEARCH_ENTRY) {
         
-#ifdef _DEBUG
-        g_printerr ("[ldap] Retrieved Key Data:\n");
-        dump_ldap_entry (lop->ldap, result);
-#endif                 
+        DEBUG_LDAP (("[ldap] Retrieved Key Data:\n"));
+        DEBUG_LDAP_ENTRY (lop->ldap, result);
+
         key = get_string_attribute (lop->ldap, result, sinfo->key_attr);
         
         if (key == NULL) {
@@ -839,7 +855,7 @@ get_key_from_ldap (SeahorseOperation *op, LDAPMessage *result)
     const gchar *fpr;
     int l;
     
-    g_return_val_if_fail (lop->ldap != NULL, FALSE);
+    g_assert (lop->ldap != NULL);
     g_assert (lop->ldap_op == -1);
     
     fingerprints = (GSList*)g_object_get_data (G_OBJECT (lop), "fingerprints");
@@ -853,7 +869,7 @@ get_key_from_ldap (SeahorseOperation *op, LDAPMessage *result)
     if (fingerprints) {
      
         fpr = (const gchar*)(fingerprints->data);
-        g_return_val_if_fail (fpr != NULL, FALSE);
+        g_assert (fpr != NULL);
 
         /* Keep track of the ones that have already been done */
         fingerprints = g_slist_next (fingerprints);
@@ -896,7 +912,7 @@ start_get_operation_multiple (SeahorseLDAPSource *lsrc, GSList *fingerprints,
     SeahorseLDAPOperation *lop;
     gpgme_error_t gerr;
     
-    g_return_val_if_fail (g_slist_length (fingerprints) > 0, NULL);
+    g_assert (g_slist_length (fingerprints) > 0);
     
     lop = seahorse_ldap_operation_start (lsrc, get_key_from_ldap, 
                                          g_slist_length(fingerprints));
@@ -920,16 +936,6 @@ start_get_operation_multiple (SeahorseLDAPSource *lsrc, GSList *fingerprints,
     return lop;
 }
 
-/* Starts a get operation for a single key */
-static SeahorseLDAPOperation *
-start_get_operation (SeahorseLDAPSource *lsrc, const gchar *fpr, 
-                     gpgme_data_t data)
-{
-    GSList *fingerprints = NULL;
-    fingerprints = g_slist_prepend (fingerprints, g_strdup (fpr));
-    return start_get_operation_multiple (lsrc, fingerprints, data);
-}
-
 /* -----------------------------------------------------------------------------
  *  SEND OPERATION 
  */
@@ -983,7 +989,7 @@ send_key_to_ldap (SeahorseOperation *op, LDAPMessage *result)
     char *values[2];
     guint l;
 
-    g_return_val_if_fail (lop->ldap != NULL, FALSE);
+    g_assert (lop->ldap != NULL);
     g_assert (lop->ldap_op == -1);
     
     keys = (GSList*)g_object_get_data (G_OBJECT (lop), "key-data");
@@ -997,7 +1003,7 @@ send_key_to_ldap (SeahorseOperation *op, LDAPMessage *result)
     if (keys) {
      
         key = (gchar*)(keys->data);
-        g_return_val_if_fail (key != NULL, FALSE);
+        g_assert (key != NULL);
 
         /* Keep track of the ones that have already been done */
         keys = g_slist_next (keys);
@@ -1042,7 +1048,7 @@ start_send_operation_multiple (SeahorseLDAPSource *lsrc, GSList *keys)
 {
     SeahorseLDAPOperation *lop;
 
-    g_return_val_if_fail (g_slist_length (keys) > 0, NULL);
+    g_assert (g_slist_length (keys) > 0);
     
     lop = seahorse_ldap_operation_start (lsrc, send_key_to_ldap, 
                                          g_slist_length (keys));
@@ -1055,15 +1061,6 @@ start_send_operation_multiple (SeahorseLDAPSource *lsrc, GSList *keys)
     return lop;
 }
 
-/* Start a key send operation */
-static SeahorseLDAPOperation *
-start_send_operation (SeahorseLDAPSource *lsrc, gchar *key)
-{
-    GSList *keys = NULL;
-    keys = g_slist_prepend (keys, key);
-    return start_send_operation_multiple (lsrc, keys);
-}
-
 /* -----------------------------------------------------------------------------
  *  SEAHORSE LDAP SOURCE
  */
@@ -1072,8 +1069,9 @@ start_send_operation (SeahorseLDAPSource *lsrc, gchar *key)
 static void seahorse_ldap_source_class_init (SeahorseLDAPSourceClass *klass);
 
 /* SeahorseKeySource methods */
-static SeahorseOperation*  seahorse_ldap_source_refresh   (SeahorseKeySource *src,
-                                                           const gchar *key);
+static SeahorseOperation*  seahorse_ldap_source_load      (SeahorseKeySource *src,
+                                                           SeahorseKeySourceLoad load,
+                                                           const gchar *match);
 static SeahorseOperation*  seahorse_ldap_source_import    (SeahorseKeySource *sksrc, 
                                                            gpgme_data_t data);
 static SeahorseOperation*  seahorse_ldap_source_export    (SeahorseKeySource *sksrc, 
@@ -1110,7 +1108,7 @@ seahorse_ldap_source_class_init (SeahorseLDAPSourceClass *klass)
     SeahorseKeySourceClass *key_class;
    
     key_class = SEAHORSE_KEY_SOURCE_CLASS (klass);
-    key_class->refresh = seahorse_ldap_source_refresh;
+    key_class->load = seahorse_ldap_source_load;
     key_class->import = seahorse_ldap_source_import;
     key_class->export = seahorse_ldap_source_export;
 
@@ -1118,49 +1116,36 @@ seahorse_ldap_source_class_init (SeahorseLDAPSourceClass *klass)
 }
 
 static SeahorseOperation*
-seahorse_ldap_source_refresh (SeahorseKeySource *src, const gchar *key)
+seahorse_ldap_source_load (SeahorseKeySource *src, SeahorseKeySourceLoad load,
+                           const gchar *match)
 {
     SeahorseOperation *op;
-    SeahorseLDAPOperation *lop;
-    gchar *pattern = NULL;
-    
-    g_return_val_if_fail (SEAHORSE_IS_KEY_SOURCE (src), NULL);
-    g_return_val_if_fail (SEAHORSE_IS_LDAP_SOURCE (src), NULL);
-    g_return_val_if_fail (key != NULL, NULL);
+    SeahorseLDAPOperation *lop = NULL;
 
-    op = parent_class->refresh (src, key);
+    g_assert (SEAHORSE_IS_KEY_SOURCE (src));
+    g_assert (SEAHORSE_IS_LDAP_SOURCE (src));
+
+    op = parent_class->load (src, load, match);
     if (op != NULL)
         return op;
     
-    /* No way to find new keys */
-    if (g_str_equal (key, SEAHORSE_KEY_SOURCE_NEW))
+    /* No way to find new or all keys */
+    if (load == SKSRC_LOAD_NEW || load == SKSRC_LOAD_ALL) 
         return seahorse_operation_new_complete (NULL);
-        
-    /* All keys, parent_class will have cleared */
-    else if(g_str_equal (key, SEAHORSE_KEY_SOURCE_ALL)) {
-        
-        g_object_get (src, "pattern", &pattern, NULL);
-        g_return_val_if_fail (pattern && pattern[0], NULL);
-    
-        lop = start_search_operation (SEAHORSE_LDAP_SOURCE (src), pattern);
-        g_return_val_if_fail (lop != NULL, NULL);
-    
-        g_free (pattern);
-    
-        seahorse_server_source_set_operation (SEAHORSE_SERVER_SOURCE (src), 
-                                              SEAHORSE_OPERATION (lop));
-        return SEAHORSE_OPERATION (lop);
+
+    /* Search for keys */
+    else if (load == SKSRC_LOAD_SEARCH)
+        lop = start_search_operation (SEAHORSE_LDAP_SOURCE (src), match);
         
     /* Load a specific key */
-    } else {
-        
-        lop = start_search_operation_fpr (SEAHORSE_LDAP_SOURCE (src), key);
-        g_return_val_if_fail (lop != NULL, NULL);
+    else if (load == SKSRC_LOAD_KEY)
+        lop = start_search_operation_fpr (SEAHORSE_LDAP_SOURCE (src), match);
     
-        seahorse_server_source_set_operation (SEAHORSE_SERVER_SOURCE (src),
-                                              SEAHORSE_OPERATION (lop));
-        return SEAHORSE_OPERATION (lop);
-    }
+    g_return_val_if_fail (lop != NULL, NULL);
+    seahorse_server_source_take_operation (SEAHORSE_SERVER_SOURCE (src),
+                                           SEAHORSE_OPERATION (lop));
+    g_object_ref (lop);
+    return SEAHORSE_OPERATION (lop);
 }
 
 static SeahorseOperation* 
@@ -1172,7 +1157,7 @@ seahorse_ldap_source_import (SeahorseKeySource *sksrc, gpgme_data_t data)
     GString *buf = NULL;
     guint len;
     
-    g_return_val_if_fail (SEAHORSE_IS_LDAP_SOURCE (sksrc), NULL);
+    g_assert (SEAHORSE_IS_LDAP_SOURCE (sksrc));
     lsrc = SEAHORSE_LDAP_SOURCE (sksrc);
     
     for (;;) {
@@ -1205,13 +1190,13 @@ seahorse_ldap_source_export (SeahorseKeySource *sksrc, GList *keys,
     SeahorseLDAPSource *lsrc;
     GSList *fingerprints = NULL;
     
-    g_return_val_if_fail (SEAHORSE_IS_LDAP_SOURCE (sksrc), NULL);
+    g_assert (SEAHORSE_IS_LDAP_SOURCE (sksrc));
     lsrc = SEAHORSE_LDAP_SOURCE (sksrc);
     
     for ( ; keys; keys = g_list_next (keys)) {
-        g_return_val_if_fail (SEAHORSE_IS_KEY (keys->data), NULL);
+        g_assert (SEAHORSE_IS_KEY (keys->data));
         fingerprints = g_slist_prepend (fingerprints,
-                g_strdup (seahorse_key_get_id (SEAHORSE_KEY (keys->data)->key)));
+                g_strdup (seahorse_key_get_keyid (SEAHORSE_KEY (keys->data))));
     }
     
     fingerprints = g_slist_reverse (fingerprints);
@@ -1224,28 +1209,19 @@ seahorse_ldap_source_export (SeahorseKeySource *sksrc, GList *keys,
 
 /**
  * seahorse_ldap_source_new
- * @locsrc: Local source to base this key source off of
- * @server: The server to connect to 
- * @pattern: The pattern to search for
+ * @uri: The server to connect to 
  * 
  * Creates a new key source for an LDAP PGP server.
  * 
  * Returns: A new LDAP Key Source
  */
 SeahorseLDAPSource*   
-seahorse_ldap_source_new (SeahorseKeySource *locsrc, const gchar *server,
-                          const gchar *pattern)
+seahorse_ldap_source_new (const gchar* uri, const gchar *host)
 {
-    SeahorseLDAPSource *lsrc;
-    
-    g_return_val_if_fail (SEAHORSE_IS_KEY_SOURCE (locsrc) && 
-                          !SEAHORSE_IS_SERVER_SOURCE (locsrc), NULL);
-    g_return_val_if_fail (server && server[0], NULL);
-    
-    lsrc = g_object_new (SEAHORSE_TYPE_LDAP_SOURCE, "local-source", locsrc,
-                         "key-server", server, "pattern", pattern, NULL);
-
-    return lsrc;  
+    g_return_val_if_fail (seahorse_ldap_is_valid_uri (uri), NULL);
+    g_return_val_if_fail (host && *host, NULL);
+    return g_object_new (SEAHORSE_TYPE_LDAP_SOURCE, "key-server", host, 
+                         "uri", uri, NULL);
 }
 
 /**
@@ -1260,6 +1236,8 @@ seahorse_ldap_is_valid_uri (const gchar *uri)
     LDAPURLDesc *url;
     int r;
     
+    g_return_val_if_fail (uri && *uri, FALSE);
+    
     r = ldap_url_parse (uri, &url);
     if (r == LDAP_URL_SUCCESS) {
         
