@@ -19,6 +19,24 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/** 
+ * SeahorsePGPSource: A key source for PGP keys retrieved from GPGME. 
+ * 
+ * - Derived from SeahorseKeySource
+ * - Since GPGME represents secret keys as seperate from public keys, this
+ *   class takes care to combine them into one logical SeahorsePGPKey object. 
+ * - Adds the keys it loads to the SeahorseContext.
+ * - Eventually a lot of stuff from seahorse-op.* should probably be merged
+ *   into this class.
+ * - Monitors ~/.gnupg for changes and reloads the keyring as necessary.
+ * 
+ * Properties:
+ *  ktype: (GQuark) The ktype (ie: SKEY_PGP) of keys originating from this 
+           key source.
+ *  location: (SeahorseKeyLoc) The location of keys that come from this 
+ *         source. (ie: SKEY_LOC_LOCAL, SKEY_LOC_REMOTE)
+ */
+ 
 #ifndef __SEAHORSE_PGP_SOURCE_H__
 #define __SEAHORSE_PGP_SOURCE_H__
 
@@ -38,17 +56,24 @@ typedef struct _SeahorsePGPSourcePrivate SeahorsePGPSourcePrivate;
 struct _SeahorsePGPSource {
     SeahorseKeySource parent;
     
+    /*< public >*/
+    gpgme_ctx_t gctx;
+    
     /*< private >*/
-    SeahorsePGPSourcePrivate *priv;
+    SeahorsePGPSourcePrivate *pv;
 };
 
 struct _SeahorsePGPSourceClass {
     SeahorseKeySourceClass parent_class;
 };
 
-SeahorsePGPSource*   seahorse_pgp_source_new    ();
+GType                seahorse_pgp_source_get_type       (void);
 
-void                 seahorse_pgp_source_load   (SeahorsePGPSource *source,
-                                                 gboolean secret_only);
+SeahorsePGPSource*   seahorse_pgp_source_new            (void);
+
+gpgme_ctx_t          seahorse_pgp_source_new_context    (SeahorsePGPSource *source);
+
+void                 seahorse_pgp_source_loadkeys       (SeahorsePGPSource *source,
+                                                         gboolean secret_only);
 
 #endif /* __SEAHORSE_PGP_SOURCE_H__ */
