@@ -20,7 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include <gnome.h>
 #include <gconf/gconf-client.h>
 #include <libgnomevfs/gnome-vfs.h>
@@ -35,13 +35,53 @@
 #include "seahorse-validity.h"
 #include "seahorse-key-manager-store.h"
 #include "seahorse-key-dialogs.h"
-#include "seahorse-key-op.h"
+#include "seahorse-pgp-key-op.h"
 #include "seahorse-key-widget.h"
 #include "seahorse-op.h"
 #include "seahorse-gpg-options.h"
 #include "seahorse-gconf.h"
+#include "seahorse-gtkstock.h"
 
-#define KEY_LIST "key_list"
+#ifdef WITH_SSH
+#include "seahorse-ssh-key.h"
+#endif
+
+/* The various tabs */
+enum KeyManagerTabs {
+    TAB_PUBLIC = 1,
+    TAB_TRUSTED,
+    TAB_PRIVATE
+};
+
+SeahorseKeyPredicate pred_public = {
+    0,                  /* ktype, set later */
+    NULL,               /* keyid */
+    SKEY_LOC_LOCAL,     /* location */
+    SKEY_PUBLIC,        /* etype */
+    0,                  /* flags */
+    SKEY_FLAG_TRUSTED,  /* nflags */
+    NULL,               /* sksrc */
+};
+
+SeahorseKeyPredicate pred_trusted = {
+    0,                  /* ktype, set later*/
+    NULL,               /* keyid */
+    SKEY_LOC_LOCAL,     /* location */
+    SKEY_PUBLIC,        /* etype */
+    SKEY_FLAG_TRUSTED,  /* flags */
+    0,                  /* nflags */
+    NULL,               /* sksrc */
+};
+
+SeahorseKeyPredicate pred_private = {
+    0,                  /* ktype, set later */
+    NULL,               /* keyid */
+    SKEY_LOC_LOCAL,     /* location */
+    SKEY_PRIVATE,       /* etype */
+    0,                  /* flags */
+    0,                  /* nflags */
+    NULL,               /* sksrc */
+};
 
 #define SEC_RING "/secring.gpg"
 #define PUB_RING "/pubring.gpg"
@@ -52,7 +92,7 @@
 static void
 quit (GtkWidget *widget, SeahorseWidget *swidget)
 {
-	seahorse_context_destroy (swidget->sctx);
+	seahorse_context_destroy (SCTX_APP());
 }
 
 /* Quits seahorse */
@@ -63,19 +103,67 @@ delete_event (GtkWidget *widget, GdkEvent *event, SeahorseWidget *swidget)
 	return TRUE;
 }
 
-/* Loads generate dialog */
-static void
-generate_activate (GtkWidget *widget, SeahorseWidget *swidget)
+/* Get the currently selected key store */
+static guint
+get_current_tab (SeahorseWidget *swidget)
 {
-	seahorse_generate_select_show (swidget->sctx);
+    GtkNotebook *notebook;
+    GtkWidget *widget;
+    guint page;
+    
+    notebook = GTK_NOTEBOOK (glade_xml_get_widget (swidget->xml, "notebook"));
+    g_return_val_if_fail (GTK_IS_NOTEBOOK (notebook), 0);
+    
+    page = gtk_notebook_get_current_page (notebook);
+    g_return_val_if_fail (page >= 0, 0);
+    
+    widget = gtk_notebook_get_nth_page (notebook, page);
+    g_return_val_if_fail (GTK_IS_WIDGET (widget), 0);
+    
+    return GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "tab-id"));
 }
-/* Loads Key generation assistant for first time users */
-static void
-new_button_clicked (GtkWidget *widget, SeahorseWidget *swidget)
+
+static GtkTreeView*
+get_current_view (SeahorseWidget *swidget)
 {
-	seahorse_generate_druid_show (swidget->sctx);
+    const gchar *name;
+
+    switch (get_current_tab (swidget)) {
+	case 0: /* Early on in initialization no tab is selected */
+		return NULL;
+    case TAB_PUBLIC:
+        name = "pub-key-list";
+        break;
+    case TAB_PRIVATE:
+        name = "sec-key-list";
+        break;
+    case TAB_TRUSTED:
+        name = "trust-key-list";
+        break;
+    default:
+        g_return_val_if_reached (NULL);
+        break;
+    };
+
+    return GTK_TREE_VIEW (glade_xml_get_widget (swidget->xml, name));
 }
 
+static GList*
+get_selected_keys (SeahorseWidget *swidget)
+{
+    GtkTreeView *view = get_current_view (swidget);
+    g_return_val_if_fail (view != NULL, NULL);
+    return seahorse_key_store_get_selected_keys (view);
+}
+
+static SeahorseKey*
+get_selected_key (SeahorseWidget *swidget, guint *uid)
+{
+    GtkTreeView *view = get_current_view (swidget);
+    g_return_val_if_fail (view != NULL, NULL);
+    return seahorse_key_store_get_selected_key (view, uid);
+}    
+
 static void
 set_numbered_status (SeahorseWidget *swidget, const gchar *t1, const gchar *t2, guint num)
 {
@@ -88,6 +176,35 @@ set_numbered_status (SeahorseWidget *swidget, const gchar *t1, const gchar *t2,
     g_free (msg);
 }
 
+/* Shows the properties for a given key */
+static void
+show_properties (SeahorseKey *skey)
+{
+    g_assert (skey);
+    
+    if (SEAHORSE_IS_PGP_KEY (skey))
+		seahorse_key_properties_new (SEAHORSE_PGP_KEY (skey));
+#ifdef WITH_SSH    
+    else if (SEAHORSE_IS_SSH_KEY (skey))
+        seahorse_ssh_key_properties_new (SEAHORSE_SSH_KEY (skey));
+#endif 
+    
+}
+
+/* Loads generate dialog */
+static void
+generate_activate (GtkWidget *widget, SeahorseWidget *swidget)
+{
+	seahorse_generate_select_show ();
+}
+
+/* Loads Key generation assistant for first time users */
+static void
+new_button_clicked (GtkWidget *widget, SeahorseWidget *swidget)
+{
+	seahorse_generate_druid_show ();
+}
+
 /* Loads import dialog */
 static void
 import_activate (GtkWidget *widget, SeahorseWidget *swidget)
@@ -98,8 +215,9 @@ import_activate (GtkWidget *widget, SeahorseWidget *swidget)
     GError *err = NULL;
     gint keys;
     
-    sksrc = seahorse_context_get_key_source (swidget->sctx);
-    g_return_if_fail (sksrc != NULL);
+    /* TODO: This needs work once we get more key types */
+    sksrc = seahorse_context_find_key_source (SCTX_APP (), SKEY_PGP, SKEY_LOC_LOCAL);
+    g_return_if_fail (sksrc && SEAHORSE_IS_PGP_SOURCE (sksrc));
     
     dialog = seahorse_util_chooser_open_new (_("Import Key"), 
                 GTK_WINDOW(glade_xml_get_widget (swidget->xml, "key-manager")));
@@ -107,7 +225,8 @@ import_activate (GtkWidget *widget, SeahorseWidget *swidget)
 
     uri = seahorse_util_chooser_open_prompt (dialog);
     if(uri) {
-        keys = seahorse_op_import_file (sksrc, uri, &err);
+        
+        keys = seahorse_op_import_file (SEAHORSE_PGP_SOURCE (sksrc), uri, &err);
         
         if (err != NULL)
             seahorse_util_handle_error (err, _("Couldn't import keys from \"%s\""), 
@@ -128,10 +247,10 @@ clipboard_received (GtkClipboard *board, const gchar *text, SeahorseWidget *swid
     GError *err = NULL;
     gint keys;
     
-    sksrc = seahorse_context_get_key_source (swidget->sctx);
-    g_return_if_fail (sksrc != NULL);
+    sksrc = seahorse_context_find_key_source (SCTX_APP (), SKEY_PGP, SKEY_LOC_LOCAL);
+    g_return_if_fail (sksrc && SEAHORSE_IS_PGP_SOURCE (sksrc));
  
-    keys = seahorse_op_import_text (sksrc, text, &err);
+    keys = seahorse_op_import_text (SEAHORSE_PGP_SOURCE (sksrc), text, &err);
  
     if (err != NULL)
         seahorse_util_handle_error (err, _("Couldn't import keys from clipboard"));
@@ -164,15 +283,15 @@ copy_activate (GtkWidget *widget, SeahorseWidget *swidget)
     GList *keys;
     guint num;
   
-    keys = seahorse_key_store_get_selected_keys (GTK_TREE_VIEW (
-                glade_xml_get_widget (swidget->xml, KEY_LIST)));
-       
+    keys = get_selected_keys (swidget);
     num = g_list_length (keys);
+    
     if (num == 0)
         return;
                
     text = seahorse_op_export_text (keys, FALSE, &err);
-
+    g_list_free (keys);
+    
     if (text == NULL)
         seahorse_util_handle_error (err, _("Couldn't export key to clipboard"));
     else {
@@ -190,12 +309,9 @@ copy_activate (GtkWidget *widget, SeahorseWidget *swidget)
 static void
 properties_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
-	SeahorseKey *skey;
-	
-	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
-		glade_xml_get_widget (swidget->xml, KEY_LIST)), NULL);
+	SeahorseKey *skey = get_selected_key (swidget, NULL);
 	if (skey != NULL)
-		seahorse_key_properties_new (swidget->sctx, skey);
+        show_properties (skey);
 }
 
 /* Loads export dialog if a key is selected */
@@ -207,8 +323,9 @@ export_activate (GtkWidget *widget, SeahorseWidget *swidget)
     GError *err = NULL;
     GList *keys;
 
-    keys = seahorse_key_store_get_selected_keys (GTK_TREE_VIEW (
-                        glade_xml_get_widget (swidget->xml, KEY_LIST)));
+    keys = get_selected_keys (swidget);
+    if (keys == NULL)
+        return;
     
     dialog = seahorse_util_chooser_save_new (_("Export Key"), 
                 GTK_WINDOW(glade_xml_get_widget (swidget->xml, "key-manager")));
@@ -278,18 +395,16 @@ backup_activate (GtkWidget *widget, SeahorseWidget *swidget)
 static void
 sign_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
-	GList *list = NULL;
-	
-	list = seahorse_key_store_get_selected_keys (GTK_TREE_VIEW (
-		glade_xml_get_widget (swidget->xml, KEY_LIST)));
-	seahorse_sign_show (swidget->sctx, list);
+    GList *keys = get_selected_keys (swidget);
+	seahorse_sign_show (keys);
+    g_list_free (keys);
 }
 
 static void
 search_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
 #ifdef WITH_KEYSERVER
-    seahorse_keyserver_search_show (swidget->sctx);
+    seahorse_keyserver_search_show ();
 #endif
 }
 
@@ -297,87 +412,93 @@ static void
 sync_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
 #ifdef WITH_KEYSERVER
-	GList *list = NULL;
-    GtkTreeView *view;
+    GList *keys = get_selected_keys (swidget);
+    GList *l;
     
-    view = GTK_TREE_VIEW (glade_xml_get_widget (swidget->xml, KEY_LIST));
-	
-	list = seahorse_key_store_get_selected_keys (view);
-    if (list == NULL)
-        list = seahorse_key_store_get_all_keys (view);
-    seahorse_keyserver_sync_show (swidget->sctx, list);
+    /* Only supported on PGP keys */
+    for (l = keys; l; l = g_list_next (l)) {
+        if (seahorse_key_get_ktype (SEAHORSE_KEY (l->data)) != SKEY_PGP) {
+            keys = l = g_list_delete_link (keys, l);
+            if (keys == NULL)
+                break;
+        }
+    }
+    
+    if (keys == NULL)
+        keys = seahorse_context_find_keys (SCTX_APP (), SKEY_PGP, SKEY_LOC_LOCAL, 0);
+    seahorse_keyserver_sync_show (keys);
+    g_list_free (keys);
 #endif
 }
 
+static void
+setup_sshkey_activate (GtkWidget *widget, SeahorseWidget *swidget)
+{
+#ifdef WITH_SSH
+    GList *keys = get_selected_keys (swidget);
+    GList *l;
+    
+    /* Only supported on SSH keys */
+    for (l = keys; l; l = g_list_next (l)) {
+        if (seahorse_key_get_ktype (SEAHORSE_KEY (l->data)) != SKEY_SSH) {
+            keys = l = g_list_delete_link (keys, l);
+            if (keys == NULL)
+                break;
+        }
+    }    
+    
+    if (keys != NULL)
+        seahorse_ssh_upload_prompt (keys);
+#endif 
+}
+
 /* Loads delete dialog if a key is selected */
 static void
 delete_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
-	GList *list = NULL;
-    GtkTreeView *view = 
-        GTK_TREE_VIEW (glade_xml_get_widget (swidget->xml, KEY_LIST));
-	
-	list = seahorse_key_store_get_selected_keys (view);
+    GList *keys = get_selected_keys (swidget);
     
     /* Special behavior for a single selection */
-    if (g_list_length (list) == 1) {
+    if (g_list_length (keys) == 1) {
         SeahorseKey *skey;
         guint uid;
         
-        skey = seahorse_key_store_get_selected_key (view, &uid);
-        if (uid > 0) 
-			/* User ids start from one in this case, strange */
-            seahorse_delete_userid_show (swidget->sctx, skey, uid + 1);
+        skey = get_selected_key (swidget, &uid);
+        if (uid > 0 && SEAHORSE_IS_PGP_KEY (skey)) 
+            /* User ids start from one in this case, strange */
+            seahorse_delete_userid_show (skey, uid + 1);
         else    
-        	seahorse_delete_show (swidget->sctx, list);
+            seahorse_delete_show (keys);
 
     /* Multiple keys */
     } else {    
-	    seahorse_delete_show (swidget->sctx, list);
+        seahorse_delete_show (keys);
     }
     
-  	g_list_free (list);
-}
-
-static void
-add_uid_activate (GtkMenuItem *item, SeahorseWidget *swidget)
-{
-	SeahorseKey *skey;
-	
-	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
-		glade_xml_get_widget (swidget->xml, KEY_LIST)), NULL);
-	if (skey != NULL)
-		seahorse_add_uid_new (swidget->sctx, skey);
-}
-
-static void
-add_revoker_activate (GtkMenuItem *item, SeahorseWidget *swidget)
-{
-	SeahorseKey *skey;
-	
-	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
-		glade_xml_get_widget (swidget->xml, KEY_LIST)), NULL);
-	if (skey != NULL)
-		seahorse_add_revoker_new (swidget->sctx, skey);
+    g_list_free (keys);
 }
 
 /* Loads preferences dialog */
 static void
 preferences_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
-	seahorse_preferences_show (swidget->sctx, 0);
+	seahorse_preferences_show (NULL);
 }
 
 static void
 expand_all_activate (GtkMenuItem *item, SeahorseWidget *swidget)
 {
-	gtk_tree_view_expand_all (GTK_TREE_VIEW (glade_xml_get_widget (swidget->xml, KEY_LIST)));
+    GtkTreeView *view = get_current_view (swidget);
+    if (view != NULL)
+	    gtk_tree_view_expand_all (view);
 }
 
 static void
 collapse_all_activate (GtkMenuItem *item, SeahorseWidget *swidget)
 {
-	gtk_tree_view_collapse_all (GTK_TREE_VIEW (glade_xml_get_widget (swidget->xml, KEY_LIST)));
+    GtkTreeView *view = get_current_view (swidget);
+    if (view != NULL)
+	    gtk_tree_view_collapse_all (view);
 }
 
 /* Shows about dialog */
@@ -386,12 +507,14 @@ about_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
 	static GtkWidget *about = NULL;
 	GdkPixbuf *pixbuf = NULL;
+    GtkWidget *tempwidget = gtk_image_new();
 
 	gchar *authors[] = {
 		"Jacob Perkins <jap1@users.sourceforge.net>",
 		"Jose Carlos Garcia Sogo <jsogo@users.sourceforge.net>",
 		"Jean Schurger <yshark@schurger.org>",
         "Nate Nielsen <nielsen@memberwebs.com>",
+        "Adam Schreiber <sadam@clemson.edu>",
         "",
         "Contributions:",
         "Albrecht Dreß <albrecht.dress@arcor.de>",
@@ -412,7 +535,12 @@ about_activate (GtkWidget *widget, SeahorseWidget *swidget)
 		return;
 	}
 
-	pixbuf = gdk_pixbuf_new_from_file (PIXMAPSDIR "seahorse.png", NULL);
+    pixbuf = gtk_widget_render_icon (tempwidget, 
+                                     SEAHORSE_STOCK_SEAHORSE, 
+                                     (GtkIconSize)-1, 
+                                     NULL); 
+    gtk_widget_destroy(tempwidget);
+
 	if (pixbuf != NULL) {
 		GdkPixbuf *temp_pixbuf = NULL;
 		
@@ -446,43 +574,72 @@ about_activate (GtkWidget *widget, SeahorseWidget *swidget)
 
 /* Loads key properties of activated key */
 static void
-row_activated (GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *arg2, SeahorseWidget *swidget)
+row_activated (GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *arg2, 
+               SeahorseWidget *swidget)
 {
+    GtkTreeView *view = get_current_view (swidget);
 	SeahorseKey *skey;
+    
+    g_return_if_fail (view != NULL);
 	
-	skey = seahorse_key_store_get_key_from_path (GTK_TREE_VIEW (glade_xml_get_widget (swidget->xml, KEY_LIST)), path, NULL);
+	skey = seahorse_key_store_get_key_from_path (view, path, NULL);
 	if (skey != NULL)
-		seahorse_key_properties_new (swidget->sctx, skey);
+        show_properties (skey);
 }
 
 static void
-selection_changed (GtkTreeSelection *selection, SeahorseWidget *swidget)
+selection_changed (GtkTreeSelection *notused, SeahorseWidget *swidget)
 {
+    GtkTreeView *view;
     GtkActionGroup *actions;
+    GList *keys, *l;
 	SeahorseKey *skey = NULL;
+    gint ktype = 0;
     gboolean selected = FALSE;
     gboolean secret = FALSE;
 	gint rows = 0;
-	
-	rows = gtk_tree_selection_count_selected_rows (selection);
-	selected = rows > 0;
+    
+    view = get_current_view (swidget);
+    if (view != NULL) {
+        GtkTreeSelection *selection = gtk_tree_view_get_selection (view);
+    	rows = gtk_tree_selection_count_selected_rows (selection);
+	    selected = rows > 0;
+    }
 	
 	if (selected) {
         set_numbered_status (swidget, _("Selected %d key"),
                                       _("Selected %d keys"), rows);
-	}
-	
-	if (rows == 1) {
-		skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (
-			glade_xml_get_widget (swidget->xml, KEY_LIST)), NULL);
-		secret = (skey != NULL && SEAHORSE_IS_KEY_PAIR (skey));
+        
+        keys = get_selected_keys (swidget);
+
+        if (keys) {
+            secret = TRUE;
+            ktype = -1;
+        }
+            
+        for (l = keys; l; l = g_list_next (l)) {
+            skey = SEAHORSE_KEY (l->data);
+            if (seahorse_key_get_etype (skey) != SKEY_PRIVATE)
+                secret = FALSE;
+            if (ktype == -1)
+                ktype = (gint)seahorse_key_get_ktype (skey);
+            else if (ktype != seahorse_key_get_ktype (skey))
+                ktype = 0;
+        }
+        
+        g_list_free (keys);
 	}
     
     actions = seahorse_widget_find_actions (swidget, "key");
     gtk_action_group_set_sensitive (actions, selected);
     
-    actions = seahorse_widget_find_actions (swidget, "keypair");
-    gtk_action_group_set_sensitive (actions, selected && secret);
+    actions = seahorse_widget_find_actions (swidget, "pgp");
+    gtk_action_group_set_sensitive (actions, ((ktype == SKEY_PGP) && (seahorse_key_get_etype (skey) != SKEY_PRIVATE)));
+    
+#ifdef WITH_SSH    
+    actions = seahorse_widget_find_actions (swidget, "ssh");
+    gtk_action_group_set_sensitive (actions, ktype == SKEY_SSH);
+#endif    
 }
 
 static void
@@ -497,7 +654,8 @@ show_context_menu (SeahorseWidget *swidget, guint button, guint32 time)
 }
 
 static gboolean
-key_list_button_pressed (GtkWidget *widget, GdkEventButton *event, SeahorseWidget *swidget)
+key_list_button_pressed (GtkWidget *widget, GdkEventButton *event, 
+                         SeahorseWidget *swidget)
 {
 	if (event->button == 3)
 		show_context_menu (swidget, event->button, event->time);
@@ -510,29 +668,13 @@ key_list_popup_menu (GtkWidget *widget, SeahorseWidget *swidget)
 {
 	SeahorseKey *skey;
 	
-	skey = seahorse_key_store_get_selected_key (GTK_TREE_VIEW (widget), NULL);
+	skey = get_selected_key (swidget, NULL);
 	if (skey != NULL)
 		show_context_menu (swidget, 0, gtk_get_current_event_time ());
        
     return FALSE;
 }
 
-
-static void
-gconf_notification (GConfClient *gclient, guint id, GConfEntry *entry, SeahorseWidget *swidget)
-{
-	const gchar *key;
-	GConfValue *value;
-	
-	key = gconf_entry_get_key (entry);
-	value = gconf_entry_get_value (entry);
-
-    /* 
-     * Removed last two gconf notification items, although I'm sure
-     * we'll get more, so leaving this code here... 
-     */
-}
-
 static void
 target_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
                            GtkSelectionData *data, guint info, guint time, SeahorseWidget *swidget)
@@ -546,20 +688,21 @@ target_drag_data_received (GtkWidget *widget, GdkDragContext *context, gint x, g
     DBG_PRINT(("DragDataReceived -->\n"));
     g_return_if_fail (data != NULL);
     
-    sksrc = seahorse_context_get_key_source (swidget->sctx);
-    g_return_if_fail (sksrc != NULL);
+    sksrc = seahorse_context_find_key_source (SCTX_APP (), SKEY_PGP, SKEY_LOC_LOCAL);
+    g_return_if_fail (sksrc && SEAHORSE_IS_PGP_SOURCE (sksrc));
     
     switch(info) {
     case TEXT_PLAIN:
-        keys = seahorse_op_import_text (sksrc, data->data, &err);
+        keys = seahorse_op_import_text (SEAHORSE_PGP_SOURCE (sksrc), 
+                                        (const gchar*)(data->data), &err);
         break;
         
     case TEXT_URIS:
-        uris = g_strsplit (data->data, "\n", 0);
+        uris = g_strsplit ((const gchar*)data->data, "\n", 0);
         for(u = uris; *u; u++) {
             g_strstrip (*u);
             if ((*u)[0]) { /* Make sure it's not an empty line */
-                keys += seahorse_op_import_file (sksrc, *u, &err);  
+                keys += seahorse_op_import_file (SEAHORSE_PGP_SOURCE (sksrc), *u, &err);  
                 if (err != NULL)
                     break;
             }
@@ -590,6 +733,26 @@ filter_changed (GtkWidget *widget, SeahorseKeyStore* skstore)
     g_object_set (skstore, "filter", text, NULL);
 }
 
+/* Clear filter when the tab changes */
+static void
+tab_changed (GtkWidget *widget, GtkNotebookPage *page, guint page_num, 
+             SeahorseWidget *swidget)
+{
+    GtkTreeView *view;
+    GtkTreeSelection *selection;
+    GtkWidget *entry = glade_xml_get_widget (swidget->xml, "filter");
+    g_return_if_fail (entry != NULL);
+    gtk_entry_set_text (GTK_ENTRY (entry), "");
+    
+    view = get_current_view (swidget);
+    if (view != NULL){
+        selection = gtk_tree_view_get_selection (view);
+        gtk_tree_selection_unselect_all (selection);
+    }
+    
+    selection_changed (NULL, swidget);
+}
+
 static void
 help_activate (GtkWidget *widget, SeahorseWidget *swidget)
 {
@@ -627,55 +790,84 @@ static GtkActionEntry ui_entries[] = {
             N_("Expand all listings"), G_CALLBACK (expand_all_activate) }, 
     { "view-collapse-all", GTK_STOCK_REMOVE, N_("_Collapse All"), NULL,
             N_("Collapse all listings"), G_CALLBACK (collapse_all_activate) }, 
+
+    { "remote-menu", NULL, N_("_Remote") },
             
     { "help-show", GTK_STOCK_HELP, N_("_Contents"), "F1",
             N_("Show Seahorse help"), G_CALLBACK (help_activate) }, 
 };
 
-static GtkActionEntry public_entries[] = {
+static GtkActionEntry key_entries[] = {
     { "key-properties", GTK_STOCK_PROPERTIES, N_("P_roperties"), NULL,
             N_("Show key properties"), G_CALLBACK (properties_activate) }, 
     { "key-export-file", GTK_STOCK_SAVE_AS, N_("E_xport..."), NULL,
             N_("Export public key"), G_CALLBACK (export_activate) }, 
     { "key-export-clipboard", GTK_STOCK_COPY, N_("_Copy Key"), "<control>C",
-            N_("Copy selected keys to the clipboard"), G_CALLBACK (copy_activate) }, 
-    { "key-sign", GTK_STOCK_INDEX, N_("_Sign..."), NULL,
-            N_("Sign public key"), G_CALLBACK (sign_activate) }, 
+            N_("Copy selected keys to the clipboard"), G_CALLBACK (copy_activate) },
     { "key-delete", GTK_STOCK_DELETE, N_("_Delete Key"), NULL,
             N_("Delete selected keys"), G_CALLBACK (delete_activate) }, 
 };
 
-static GtkActionEntry private_entries[] = {
-    { "key-add-userid", GTK_STOCK_ADD, N_("Add _User ID..."), NULL,
-            N_("Add a new user ID"), G_CALLBACK (add_uid_activate) }, 
-    { "key-add-revoker", GTK_STOCK_CANCEL, N_("Add _Revoker..."), NULL,
-            N_("Add the default key as a revoker"), G_CALLBACK (add_revoker_activate) }, 
+static GtkActionEntry pgp_entries[] = {
+    { "key-sign", GTK_STOCK_INDEX, N_("_Sign..."), NULL,
+            N_("Sign public key"), G_CALLBACK (sign_activate) }, 
 };
 
-static GtkActionEntry remote_entries[] = {
-    { "remote-menu", NULL, N_("_Remote") },
+static GtkActionEntry ssh_entries[] = {
+    { "remote-upload-ssh", NULL, N_("Configure SSH Key for Computer..."), "",
+            N_("Send public SSH key to another machine, and enable logins using that key."), G_CALLBACK (setup_sshkey_activate) },
+};
+
+static GtkActionEntry keyserver_entries[] = {
     { "remote-find", GTK_STOCK_FIND, N_("_Find Remote Keys..."), "",
             N_("Search for keys on a key server"), G_CALLBACK (search_activate) }, 
     { "remote-sync", GTK_STOCK_REFRESH, N_("_Sync and Publish Keys..."), "",
             N_("Publish and/or sync your keys with those online."), G_CALLBACK (sync_activate) }, 
 };
 
+void
+initialize_tab (SeahorseWidget *swidget, const gchar *tabwidget, guint tabid, 
+                const gchar *viewwidget, SeahorseKeyPredicate *pred)
+{
+    SeahorseKeyset *skset;
+    SeahorseKeyStore *skstore;
+    GtkTreeSelection *selection;
+    GtkTreeView *view;
+    GtkWidget *tab;
+    
+    skset = seahorse_keyset_new_full (pred);
+    
+    tab = glade_xml_get_widget (swidget->xml, tabwidget);
+    g_return_if_fail (tab != NULL);
+    g_object_set_data (G_OBJECT (tab), "tab-id", GINT_TO_POINTER (tabid));
+    
+    /* init key list & selection settings */
+    view = GTK_TREE_VIEW (glade_xml_get_widget (swidget->xml, viewwidget));
+    g_return_if_fail (view != NULL);
+    
+    selection = gtk_tree_view_get_selection (view);
+    gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
+    g_signal_connect (selection, "changed", G_CALLBACK (selection_changed), swidget);
+    
+    /* Add new key store and associate it */
+    skstore = seahorse_key_manager_store_new (skset, view);
+    g_object_set_data_full (G_OBJECT (view), "key-store", skstore, g_object_unref);
+    
+    /* For the filtering */
+    glade_xml_signal_connect_data(swidget->xml, "on_filter_changed",
+                                  G_CALLBACK(filter_changed), skstore);
+}
+
 GtkWindow* 
-seahorse_key_manager_show (SeahorseContext *sctx)
+seahorse_key_manager_show (SeahorseOperation *op)
 {
     GtkWindow *win;
-    SeahorseOperation *operation;
 	SeahorseWidget *swidget;
-	GtkTreeView *view;
     GtkWidget *w;
-	GtkTreeSelection *selection;
-    SeahorseKeyStore *skstore;
-    SeahorseKeySource *sksrc;
     GtkActionGroup *actions;
     GtkAction *action;
 	
-	swidget = seahorse_widget_new ("key-manager", sctx);
-	gtk_object_sink (GTK_OBJECT (sctx));
+	swidget = seahorse_widget_new ("key-manager");
     win = GTK_WINDOW (glade_xml_get_widget (swidget->xml, "key-manager"));
     
     /* General normal actions */
@@ -686,8 +878,8 @@ seahorse_key_manager_show (SeahorseContext *sctx)
     
     /* Actions that are allowed on all keys */
     actions = gtk_action_group_new ("key");
-    gtk_action_group_add_actions (actions, public_entries, 
-                                  G_N_ELEMENTS (public_entries), swidget);
+    gtk_action_group_add_actions (actions, key_entries, 
+                                  G_N_ELEMENTS (key_entries), swidget);
     seahorse_widget_add_actions (swidget, actions);
 
     /* Mark the properties toolbar button as important */
@@ -695,23 +887,32 @@ seahorse_key_manager_show (SeahorseContext *sctx)
     g_return_val_if_fail (action, win);
     g_object_set (action, "is-important", TRUE, NULL);
 
-    /* Actions for public keys */
-    actions = gtk_action_group_new ("keypair");
-    gtk_action_group_add_actions (actions, private_entries,
-                                  G_N_ELEMENTS (private_entries), swidget);
+    /* Actions for pgp keys */
+    actions = gtk_action_group_new ("pgp");
+    gtk_action_group_add_actions (actions, pgp_entries,
+                                  G_N_ELEMENTS (pgp_entries), swidget);
+    seahorse_widget_add_actions (swidget, actions);                              
+    
+    /* Actions for SSH keys */
+    actions = gtk_action_group_new ("ssh");
+    gtk_action_group_add_actions (actions, ssh_entries,
+                                  G_N_ELEMENTS (ssh_entries), swidget);
     seahorse_widget_add_actions (swidget, actions);
-
-    /* Actions for keyservers */
-#ifdef WITH_KEYSERVER      
-    actions = gtk_action_group_new ("remote");
-    gtk_action_group_add_actions (actions, remote_entries, 
-                                  G_N_ELEMENTS (remote_entries), swidget);
-    seahorse_widget_add_actions (swidget, actions);                                  
+	
+#ifndef WITH_SSH
+    gtk_action_group_set_visible (actions, FALSE);
 #endif
 
-    sksrc = seahorse_context_get_key_source (sctx);
-    g_return_val_if_fail (sksrc != NULL, win);
-    	
+    /* Actions for keyservers */
+    actions = gtk_action_group_new ("keyserver");
+    gtk_action_group_add_actions (actions, keyserver_entries, 
+                                  G_N_ELEMENTS (keyserver_entries), swidget);
+    seahorse_widget_add_actions (swidget, actions);                                  
+	
+#ifndef WITH_KEYSERVER
+	gtk_action_group_set_visible (actions, FALSE);
+#endif
+ 	
 	/* close event */
 	glade_xml_signal_connect_data (swidget->xml, "quit_event",
 		G_CALLBACK (delete_event), swidget);
@@ -724,52 +925,46 @@ seahorse_key_manager_show (SeahorseContext *sctx)
 	glade_xml_signal_connect_data (swidget->xml, "key_list_popup_menu",
 		G_CALLBACK (key_list_popup_menu), swidget);
         
-	/* init gclient */
-	seahorse_gconf_notify_lazy (UI_SCHEMAS, (GConfClientNotifyFunc) gconf_notification, 
-                                swidget, GTK_WIDGET (win));
-	
 	/* first time signals */
 	glade_xml_signal_connect_data (swidget->xml, "import_button_clicked",
 		G_CALLBACK (import_activate), swidget);
 	glade_xml_signal_connect_data (swidget->xml, "new_button_clicked",
 		G_CALLBACK (new_button_clicked), swidget);
-		
-	/* init key list & selection settings */
-	view = GTK_TREE_VIEW (glade_xml_get_widget (swidget->xml, KEY_LIST));
-	selection = gtk_tree_view_get_selection (view);
-	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
-	g_signal_connect (selection, "changed",
-		G_CALLBACK (selection_changed), swidget);
-    skstore = seahorse_key_manager_store_new (sksrc, view);
-	selection_changed (selection, swidget);
 
+    /* The notebook */
+    glade_xml_signal_connect_data (swidget->xml, "on_tab_changed", 
+                                  G_CALLBACK(tab_changed), swidget);    
+    
+    /* Initialize the tabs, and associate them up */
+    initialize_tab (swidget, "pub-key-tab", TAB_PUBLIC, "pub-key-list", &pred_public);
+    initialize_tab (swidget, "trust-key-tab", TAB_TRUSTED, "trust-key-list", &pred_trusted);
+    initialize_tab (swidget, "sec-key-tab", TAB_PRIVATE, "sec-key-list", &pred_private);
+    
+    /* Set focus to the current key list */
+    w = GTK_WIDGET (get_current_view (swidget));
+    gtk_widget_grab_focus (w);
+    
     /* To avoid flicker */
     seahorse_widget_show (swidget);
-    gtk_widget_grab_focus (GTK_WIDGET (view));	
-
+	
     /* Setup drops */
     gtk_drag_dest_set (GTK_WIDGET (win), GTK_DEST_DEFAULT_ALL, 
                 seahorse_target_entries, seahorse_n_targets, GDK_ACTION_COPY);
     gtk_signal_connect (GTK_OBJECT (win), "drag_data_received",
                 GTK_SIGNAL_FUNC (target_drag_data_received), swidget);
                         
-    /* For the filtering */
-    glade_xml_signal_connect_data(swidget->xml, "on_filter_changed",
-                G_CALLBACK(filter_changed), skstore);
-
     /* Hook progress bar in */
-    operation = seahorse_key_source_get_operation (sksrc);
-    g_return_val_if_fail (operation != NULL, win);
-    
     w = glade_xml_get_widget (swidget->xml, "status");
-    seahorse_progress_appbar_set_operation (w, operation);
+    seahorse_progress_appbar_set_operation (w, op);
 
     /* Although not all the keys have completed we'll know whether we have 
      * any or not at this point */
-	if (seahorse_key_source_get_count (sksrc, FALSE) == 0) {
+	if (seahorse_context_get_count (SCTX_APP()) == 0) {
 		w = glade_xml_get_widget (swidget->xml, "first-time-box");
 		gtk_widget_show (w);
 	}
     
+	selection_changed (NULL, swidget);
+    
     return win;
 }
